/* Copyright (c) 2018 Atmark Techno, Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <modbus.h>

#define SERIAL_SPEED 19200
#define SERIAL_PARITY 'N'
#define SERIAL_DATABIT 8
#define SERIAL_STOPBIT 1

int main(int argc, char *argv[])
{
	static uint8_t raw_req[] = { 0x01, 0x05, 0x00, 0x03, 0x00, 0x00 }; /* 送信パケット */
	uint8_t rsp[MODBUS_TCP_MAX_ADU_LENGTH];
	int ret = EXIT_SUCCESS;
	int raw_req_size = sizeof (raw_req) / sizeof (raw_req[0]);
	int req_length;
	int rsp_length;
	modbus_t *ctx;

	if (argc <= 2) {
		printf("Usage: %s <device> <on/off>\n\n", argv[0]);
		return EXIT_SUCCESS;
	}

	if (!strcmp(argv[2], "on")) {
		raw_req[4] = 0xFF;
	} else if (!strcmp(argv[2], "off")) {
		raw_req[4] = 0x00;
	} else {
		printf("Usage: %s <device> <on/off>\n\n", argv[0]);
		return EXIT_SUCCESS;
	}

	ctx = modbus_new_rtu(argv[1], SERIAL_SPEED, SERIAL_PARITY, SERIAL_DATABIT, SERIAL_STOPBIT);
	if (ctx == NULL) {
		fprintf(stderr, "Unable to allocate libmodbus context\n"
			"modbus error\n");
		return EXIT_FAILURE;
	}

	modbus_set_debug(ctx, TRUE);
	modbus_set_error_recovery(ctx, MODBUS_ERROR_RECOVERY_LINK | MODBUS_ERROR_RECOVERY_PROTOCOL );
	if (modbus_connect(ctx) == -1) {
		fprintf(stderr, "Connection failed: %s\n"
			"modbus error\n",
			modbus_strerror(errno));
		modbus_free(ctx);
		return EXIT_FAILURE;
	}

	req_length = modbus_send_raw_request(ctx, raw_req, raw_req_size);
	if (req_length < 0 ) { 
		fprintf(stderr, "faild: %s\n"
			"modbus error\n",
			modbus_strerror(errno));
		ret = EXIT_FAILURE;
	}

	rsp_length = modbus_receive_confirmation(ctx, rsp);
        if (rsp_length < 0 ) { 
                fprintf(stderr, "faild: %s\n"
                        "modbus error\n",
                        modbus_strerror(errno));
                ret = EXIT_FAILURE;
        }

	modbus_close(ctx);
	modbus_free(ctx);

	return ret;
}
